#include <avr/sleep.h>
#include <avr/wdt.h>
#define SYSCLOCK F_CPU
#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>

// The ID of the station is wired into the code, set this between 0-0xFFFF to identify your station uniquely
#define STATION_ID	0x1234

// If defined, DHT sensor is not present and debug values are sent instead of DHT sensor values
//#define NODHT	1

#define DHTPIN            2         // Pin which is connected to the DHT sensor.
#define DHTTYPE           DHT22     // DHT 22 (AM2302)

const byte LED = 13;
int state = 0;
bool lofreq = false;
int  stateCtr = 4;
unsigned long Extent;

#define STATE_SENDID	1
#define STATE_SENDTEMP	2
#define STATE_SENDHUM	3

#define TOPBIT 0x80000000

#define IR_SEND_FREQ		38	// Freq in kHz for the "signal present" state
//#define IR_SEND_NOFREQ		34	// Freq in kHz for the "no signal present" state
#define IR_SEND_NOFREQ		48	// Freq in kHz for the "no signal present" state

#define IR_SEND_PWM_START     (TCCR2A |= _BV(COM2B1))
#define IR_SEND_MARK_TIME(time)  My_delay_uSecs(time)
#define IR_SEND_PWM_STOP    (TCCR2A &= ~(_BV(COM2B1)))
#define IR_SEND_CONFIG_KHZ(val) ({ \
	const uint8_t pwmval = SYSCLOCK / 2000 / (val); \
	TCCR2A = _BV(WGM20);  TCCR2B = _BV(WGM22) | _BV(CS20); \
	OCR2A = pwmval; OCR2B = pwmval / 6; })
//#define LIGHT_SET_KHZ(val) ({})
	#define LIGHT_SET_KHZ(val) ({ \
		const uint8_t pwmval = SYSCLOCK / 2000 / (val); \
		OCR2A = pwmval; OCR2B = pwmval / 6; })
#define IR_SEND_PWM_PIN	3


DHT_Unified dht(DHTPIN, DHTTYPE);
  
// watchdog interrupt
ISR (WDT_vect) 
{
   wdt_disable();  // disable watchdog
}  // end of WDT_vect
 
//The Arduino built in function delayMicroseconds has limits we wish to exceed
//Therefore we have created this alternative
void  My_delay_uSecs(unsigned int T) {
  if(T){if(T>16000) {delayMicroseconds(T % 1000); delay(T/1000); } else delayMicroseconds(T);};
}

 
void markLight(unsigned int time) {
 LIGHT_SET_KHZ(IR_SEND_FREQ);
 My_delay_uSecs(time);
 Extent+=time;
}

void spaceLight(unsigned int time) {
 LIGHT_SET_KHZ(IR_SEND_NOFREQ);
 My_delay_uSecs(time);
 Extent+=time;
}

void sendLight(unsigned long data, unsigned char Num_Bits, unsigned int Head_Mark, unsigned int Head_Space, 
                             unsigned int Mark_One, unsigned int Mark_Zero, unsigned int Space_One, unsigned int Space_Zero, 
							 bool Use_Stop, unsigned long Max_Extent) {
  Extent=0;
  data = data << (32 - Num_Bits);
  if(Head_Mark) markLight(Head_Mark); 
  if(Head_Space) spaceLight(Head_Space);
  for (int i = 0; i <Num_Bits; i++) {
    if (data & TOPBIT) {
      markLight(Mark_One);  spaceLight(Space_One);
    } 
    else {
      markLight(Mark_Zero);  spaceLight(Space_Zero);
    }
    data <<= 1;
  }
  if(Use_Stop) 
	markLight(Mark_One);   //stop bit of "1"
  if(Max_Extent) {
#ifdef IRLIB_TRACE
    Serial.print("Max_Extent="); Serial.println(Max_Extent);
    Serial.print("Extent="); Serial.println(Extent);
    Serial.print("Difference="); Serial.println(Max_Extent-Extent);
#endif
    spaceLight(Max_Extent-Extent); 
  }
  else 
	  spaceLight(Space_One);

};


void sendLightData(unsigned long data)
{
//void IRsendBase::sendGeneric(unsigned long data, unsigned char Num_Bits, unsigned int Head_Mark, unsigned int Head_Space, 
//                             unsigned int Mark_One, unsigned int Mark_Zero, unsigned int Space_One, unsigned int Space_Zero, 
//							 unsigned char kHz, bool Use_Stop, unsigned long Max_Extent)
// Num_Bits: 	32
// All the delays are in microsecs
// Head_Mark: 	564*16
// Head_Space:	564*8
// Mark_One:	564
// Mark_Zero:	564
// Space_One:	564*3
// Space_Zero:	564
// kHz: 38
// Use_Stop:	true
// Max_Extent:	0	
//    sendGeneric(data,32, 564*16, 564*8, 564, 564, 564*3, 564, 38, true);
//    sendGeneric(data,16, 105*4, 105*4, 105, 105, 105*2, 105, 38, true);
//    int min_burst = 270;	// minimum burst: 10 cycles, 260 microsec
//    int min_gap = 370; 		// minimum gap: 14 cycles, 364 microsec
//    sendLight(data,20, min_burst*2, min_gap*2, min_burst, min_burst, min_gap*3, min_gap, 38, true);
    sendLight(data,20, 564*16, 564*8, 564, 564, 564*3, 564, 38, true);
};
 
void setup () { 
  Serial.begin(9600); 
  Serial.println("Light sender starts");
  pinMode (LED, OUTPUT);
  digitalWrite (LED, HIGH);
  pinMode(IR_SEND_PWM_PIN, OUTPUT);
  digitalWrite (IR_SEND_PWM_PIN, LOW);   
  IR_SEND_CONFIG_KHZ(IR_SEND_NOFREQ);
  IR_SEND_PWM_START;
#ifndef NODHT
  delay(2000);		// DHT-22 warmup period
  dht.begin();
#endif
  state = STATE_SENDID;
  Serial.println("Init finished");
  lofreq = false;
  stateCtr = 8;
  digitalWrite (LED, LOW);
}

void loop () 
{
  unsigned long selector, data;
  int nextstate;
  sensors_event_t event;
  bool error = false;  

  digitalWrite (LED, HIGH);
  nextstate =  STATE_SENDID;
  selector = 0;
  data = 0;
  switch( state ) {
	  case STATE_SENDID:
		selector = STATE_SENDID;
		data = (unsigned long)STATION_ID;
//		nextstate = STATE_SENDID;
		nextstate = STATE_SENDTEMP;
		break;
		
	  case STATE_SENDTEMP:
		selector = STATE_SENDTEMP;
#ifdef NODHT
		data = 22;
#else		
		dht.temperature().getEvent(&event);
		if (isnan(event.temperature)) {
			Serial.println("Error reading temperature!");
			error = true;
		}
		else {
			Serial.print("Temperature: ");
			Serial.print(event.temperature);
			Serial.println(" *C");
			data = (unsigned long)( ( event.temperature * 100.0 ) + 10000.0 );
		}
#endif
		nextstate = STATE_SENDHUM;
		break;

	  case STATE_SENDHUM:
		selector = STATE_SENDHUM;
#ifdef NODHT
		data = 52;
#else
		dht.humidity().getEvent(&event);
		if (isnan(event.relative_humidity)) {
			Serial.println("Error reading humidity!");
			error = true;
		}
		else {
			Serial.print("Humidity: ");
			Serial.print(event.relative_humidity);
			Serial.println("%");
			data = (unsigned long)( event.relative_humidity * 100.0 );
		}
#endif
		nextstate = STATE_SENDID;
		break;
  }
  data &= 0xFFFF;
  data |= selector << 16;  

  if( error )
	  Serial.println( "Not sending due to sensor reading error" );
  else {
	sendLightData( data);
	Serial.print("Sent: ");
	Serial.println( data,HEX );
  }
  state = nextstate;

  /*
  if( lofreq ) {
	  LIGHT_SET_KHZ(IR_SEND_FREQ);
	  Serial.println( "SEND_FREQ" );
  }
  else {
	  LIGHT_SET_KHZ(IR_SEND_NOFREQ);
	  Serial.println( "SEND_NOFREQ" );
  }
  lofreq = !lofreq;
*/
/*
   --stateCtr;
   Serial.println( "TICK" );
   if( stateCtr <= 0 ) {
	   if( lofreq ) {
		digitalWrite (IR_SEND_PWM_PIN, LOW);   
	   } else {
		digitalWrite (IR_SEND_PWM_PIN, HIGH);   
	   }
	   lofreq = !lofreq;
	   stateCtr = 8;
   }
*/
   Serial.flush();
   digitalWrite (LED, LOW);
   delay(1000);
/*
  // disable ADC
  ADCSRA = 0;  

  // clear various "reset" flags
  MCUSR = 0;     
  // allow changes, disable reset
  WDTCSR = bit (WDCE) | bit (WDE);
  // set interrupt mode and an interval 
  WDTCSR = bit (WDIE) | bit (WDP2) | bit (WDP1);   // set WDIE, 128K prescaler (about 1 sec delay)
  wdt_reset();  // pat the dog
  
  set_sleep_mode (SLEEP_MODE_PWR_DOWN);  
  noInterrupts ();           // timed sequence follows
  sleep_enable();
 
  // turn off brown-out enable in software
  MCUCR = bit (BODS) | bit (BODSE);
  MCUCR = bit (BODS); 
  interrupts ();             // guarantees next instruction executed
  sleep_cpu ();  
  
  // cancel sleep as a precaution
  sleep_disable();
  */
} // end of loop

